/* bmploaderGLUT.c
 * Programme affichant une image BMP 8 ou 24 bits non compresse.
 * Touches:
 *	ESC - Quitter
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 21 aot 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "bmp.h"

/* Caractristiques de la fentre et de l'cran */
#define TITLE "BMPLoaderGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;

/* Image BMP */
static BMPFILE myBMP;	

/* Position de l'image */
static int x, y;

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}	
	init();
	glutDisplayFunc(glutDisplay);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tESC - Quitter\n");
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	
	glShadeModel(GL_SMOOTH);
	
	// Chargement de l'image
	if (LoadBMPFile("image.bmp", &myBMP) != 1)
		printf("erreur chargement BMP\n");
	
	if (!g_fullscreen) {	
		glutReshapeWindow(myBMP.imageWidth, myBMP.imageHeight);
	}
}
		
/* Affichage et animation */
void glutDisplay(void)
{	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();	
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
	glRasterPos2i(x, y);		// position
	// dessiner l'image
	glDrawPixels(myBMP.imageWidth, myBMP.imageHeight, myBMP.texFormat
			, GL_UNSIGNED_BYTE, myBMP.imageData);
		
	glFlush();
	glutSwapBuffers();
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0f, w - 1.0, 0.0, h - 1.0, -1.0, 1.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// calcul la position pour afficher l'image
	// au centre de la fentre ou de l'cran
	if (!g_fullscreen) {
		y = glutGet(GLUT_WINDOW_HEIGHT)/2 - myBMP.imageHeight/2;
		x = glutGet(GLUT_WINDOW_WIDTH)/2 - myBMP.imageWidth/2;
	}
	else {
		y = glutGet(GLUT_SCREEN_HEIGHT)/2 - myBMP.imageHeight/2;
		x = glutGet(GLUT_SCREEN_WIDTH)/2 - myBMP.imageWidth/2;
	}
	if (x < 0) x = 0;
	if (y < 0) y = 0;
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}



